/* Timer.h documents a simple, platform-independent interval timer
 *  that measures CPU usage in (fractional) seconds. 
 *  
 * Author: Joel Adams, for Hands On C++.
 * Date:   November 1997
 * Modification history:
 *      March 1998: ANSI compliance, interface improvements by J. Adams
 *      February 1998: Seconds() added by Keith Vander Linden
 *      July 2003: renamed some methods by Jeremy D. Frens
 */

#ifndef TIMER
#define TIMER

#include <iostream>   // ostream
#include <ctime>      // C time library: clock_t, clock(), CLOCKS_PER_SEC
using namespace std;

class Timer
{
public:
  Timer();
  void start();
  void stop();
  void reset();
  double getSeconds() const;
  void print(ostream & out) const;

private:
  clock_t myStartTime;
  clock_t myRunTime;
  bool running;
};

inline Timer::Timer()
{
  myStartTime = myRunTime = 0;
  running = false;
}

inline void Timer::start()
{
  running = true;
  myStartTime = clock();
}

inline void Timer::stop()
{
  if (running)
    {
      myRunTime += clock() - myStartTime;
      running = false;
    }
}

inline void Timer::reset()
{
  myRunTime = 0;
  myStartTime = clock();
}

inline double Timer::getSeconds() const
{
  if (running)
    return double(clock() - myStartTime) / double(CLOCKS_PER_SEC);
  else
    return double(myRunTime) / double(CLOCKS_PER_SEC);
}

inline void Timer::print(ostream & out) const
{
  out << getSeconds();
}

inline ostream & operator<< (ostream & out, const Timer & aTimer)
{
  aTimer.print(out);
  return out;
}

#endif
